@@ -2,7 +2,7 @@ package DBIx::XHTML_Table;
 
 use strict;
 use warnings;
-our $VERSION = '1.41';
+our $VERSION = '1.42';
 
 use DBI;
 use Carp;
@@ -67,7 +67,7 @@ sub exec_query {
 	carp $@ and return undef if $@;
 
 	# store the results
-	$self->{'fields_arry'} = [ map { lc }         @{$self->{'sth'}->{'NAME'}} ];
+	$self->{'fields_arry'} = [ @{$self->{'sth'}->{'NAME'}} ];
 	$self->{'fields_hash'} = $self->_reset_fields_hash();
 	$self->{'rows'}        = $self->{'sth'}->fetchall_arrayref();
 	carp "can't call exec_query(): no data was returned from query" unless @{$self->{'rows'}};
@@ -142,7 +142,23 @@ sub map_cell {
 	carp "map_cell() is being ignored - no data" and return $self unless $self->{'rows'};
 
 	$cols = $self->_refinate($cols);
-	$self->{'map_cell'}->{$_} = $sub for @$cols;
+    for (@$cols) {
+        my $key;
+        if (defined $self->{'fields_hash'}->{$_}) {
+            $key = $_;
+        } elsif( defined $self->{'fields_hash'}->{lc $_}) {
+            $key = lc $_;
+        } else {
+            SEARCH: for my $k (sort keys %{ $self->{'fields_hash'} }) {
+                if (lc( $k ) eq lc( $_ )) {
+                    $key = $k;
+                    last SEARCH;
+                }
+            }
+        }
+        next unless $key;
+        $self->{'map_cell'}->{$key} = $sub;
+    }
 	return $self;
 }
 
@@ -152,7 +168,23 @@ sub map_head {
 	carp "map_head() is being ignored - no data" and return $self unless $self->{'rows'};
 
 	$cols = $self->_refinate($cols);
-	$self->{'map_head'}->{$_} = $sub for @$cols;
+    for (@$cols) {
+        my $key;
+        if (defined $self->{'fields_hash'}->{$_}) {
+            $key = $_;
+        } elsif( defined $self->{'fields_hash'}->{lc $_}) {
+            $key = lc $_;
+        } else {
+            SEARCH: for my $k (sort keys %{ $self->{'fields_hash'} }) {
+                if (lc( $k ) eq lc( $_ )) {
+                    $key = $k;
+                    last SEARCH;
+                }
+            }
+        }
+        next unless $key;
+        $self->{'map_head'}->{$key} = $sub;
+    }
 
 	return $self;
 }
@@ -166,13 +198,29 @@ sub add_col_tag {
 }
 
 sub calc_totals {
-
 	my ($self,$cols,$mask) = @_;
 	return undef unless $self->{'rows'};
 
 	$self->{'totals_mask'} = $mask;
 	$cols = $self->_refinate($cols);
-	my @indexes = map { $self->{'fields_hash'}->{lc $_} } @$cols;
+
+	my @indexes;
+    for (@$cols) {
+        my $index;
+        if (exists $self->{'fields_hash'}->{$_}) {
+            $index = $self->{'fields_hash'}->{$_};    
+        } elsif (exists $self->{'fields_hash'}->{lc $_}) {
+            $index = $self->{'fields_hash'}->{lc $_};    
+        } else {
+            SEARCH: for my $k (sort keys %{ $self->{'fields_hash'} }) {
+                if (lc( $k ) eq lc( $_ )) {
+                    $index = $self->{'fields_hash'}->{$k};
+                    last SEARCH;
+                }
+            }
+        }
+        push @indexes, $index;
+    }
 
 	$self->{'totals'} = $self->_total_chunk($self->{'rows'},\@indexes);
 
@@ -180,14 +228,29 @@ sub calc_totals {
 }
 
 sub calc_subtotals {
-
 	my ($self,$cols,$mask,$nodups) = @_;
-
 	return undef unless $self->{'rows'};
 
 	$self->{'subtotals_mask'} = $mask;
 	$cols = $self->_refinate($cols);
-	my @indexes = map { $self->{'fields_hash'}->{lc $_} } @$cols;
+
+	my @indexes;
+    for (@$cols) {
+        my $index;
+        if (exists $self->{'fields_hash'}->{$_}) {
+            $index = $self->{'fields_hash'}->{$_};    
+        } elsif (exists $self->{'fields_hash'}->{lc $_}) {
+            $index = $self->{'fields_hash'}->{lc $_};    
+        } else {
+            SEARCH: for my $k (sort keys %{ $self->{'fields_hash'} }) {
+                if (lc( $k ) eq lc( $_ )) {
+                    $index = $self->{'fields_hash'}->{$k};
+                    last SEARCH;
+                }
+            }
+        }
+        push @indexes, $index;
+    }
 
 	my $beg = 0;
 	foreach my $end (@{$self->{'body_breaks'}}) {
@@ -248,10 +311,25 @@ sub set_col_colors {
 sub set_group {
 	my ($self,$group,$nodup,$value) = @_;
 	$self->{'nodup'} = $value || $self->{'null_value'} if $nodup;
-	$self->{'group'} = $group = $group =~ /^\d+$/ 
-		? $self->_lookup_name($group) || lc $group : lc $group;
 
-	my $index = $self->{'fields_hash'}->{$group} || 0;
+    my $index;
+    if ($group =~ /^\d+$/) {
+        $index = $group;
+    } elsif (exists $self->{'fields_hash'}->{$group}) {
+        $index = $self->{'fields_hash'}->{$group};    
+	    $self->{'group'} = $group;
+    } elsif (exists $self->{'fields_hash'}->{lc $group}) {
+        $index = $self->{'fields_hash'}->{lc $group};    
+	    $self->{'group'} = lc $group;
+    } else {
+        SEARCH: for my $k (sort keys %{ $self->{'fields_hash'} }) {
+            if (lc( $k ) eq lc( $group )) {
+                $index = $self->{'fields_hash'}->{$k};
+	            $self->{'group'} = $k;
+                last SEARCH;
+            }
+        }
+    }
 
 	# initialize the first 'repetition'
 	my $rep = $self->{'rows'}->[0]->[$index];
@@ -274,7 +352,7 @@ sub set_pk {
 	my $pk   = shift || 'id';
 	$pk = $pk =~ /^\d+$/ ? $self->_lookup_name($pk) || $pk : $pk;
 	carp "can't call set_pk(): too late to set primary key" if exists $self->{'rows'};
-	$self->{'pk'} = lc $pk;
+	$self->{'pk'} = $pk;
 
 	return $self;
 }
@@ -444,8 +522,8 @@ sub _build_head_row {
 		if (my $sub = $self->{'map_head'}->{$field}) {
 			$field = $sub->($field);
 		}
-		else {
-			$field = ucfirst $field unless $self->{'no_ucfirst'};
+		elsif (!$self->{'no_ucfirst'}) {
+			$field = ucfirst( lc( $field ) );
 		}
 
         # bug 21761 "Special XML characters should be expressed as entities"
@@ -677,6 +755,7 @@ sub _rotate {
 # always returns an array ref
 sub _refinate {
 	my ($self,$ref) = @_;
+    $ref = undef if ref($ref) eq 'ARRAY' && scalar( @$ref ) < 1;
 	$ref = [@{$self->{'fields_arry'}}] unless defined $ref;
 	$ref = [$ref] unless ref $ref eq 'ARRAY';
 	return [map {$_ =~ /^\d+$/ ? $self->_lookup_name($_) || $_ : $_} @$ref];
@@ -711,7 +790,7 @@ sub _reset_fields_hash {
 sub _do_black_magic {
 	my ($self,$ref,$headers) = @_;
     croak "bad data" unless ref( $ref->[0] ) eq 'ARRAY';
-	$self->{'fields_arry'} = $headers ? [@$headers] : [ map { lc } @{ shift @$ref } ];
+	$self->{'fields_arry'} = $headers ? [@$headers] : [ @{ shift @$ref } ];
 	$self->{'fields_hash'} = $self->_reset_fields_hash();
 	$self->{'rows'}        = $ref;
 }
